/******************************************************************************
		Neville's Iterated Interpolation - Algorithm 3.1
*******************************************************************************

To evaluate the interplating polynomial P on the (n+1) distinct numbers x0,
..., xn at the number x for the function f():

INPUT numbers x0, x1,..., xn; values f(x0), f(x1),.., f(xn) as the first column
Q(0,0), Q(1,0),..., Q(n,0) of Q.

OUTPUT the table Q with P(x) = Q(n,n).

*******************************************************************************
*	Written by:  Harold A. Toomey, BYU, Summer 1989, v4.11		      *
******************************************************************************/

#include <stdio.h>		/* for scanf() and printf() */
#include <math.h>		/* For math functions */
#include "naa_util.h"		/* Numerical Analysis Algorithms Utilities. */
#define	PI	3.14159265358979323846264338327950288419716939937510582097


main()
{
  double **Q, *x, X, f();
  int i, j, n, ch;

  get_title();

  printf("Neville's Iterated Interpolation - Algorithm 3.1\n\n");
  fprintf(stderr, "Neville's Iterated Interpolation - Algorithm 3.1\n\n");

  fprintf(stderr, "Enter number of data points, n+1: ");
  scanf("%d", &n);
  if (n <= 0)
    naaerror("n must be > 0.");	/* ERROR. */
  printf("Interpolating polynomial is of degree %d.\n", --n);

  fprintf(stderr, "Enter the x to evaluate: ");
  scanf("%lf", &X);
  printf("Evaluating P(x) at x = %lg\n\n", X);

  Q = dmatrix(0,n,0,n);
  x = dvector(0,n);

  fprintf(stderr, "Does f(x) need to be evaluated? (y/n): ");
  ch = getchar();		/* filters out New Line/Carrage Return */
  ch = tolower(getchar());
  if (ch == 'y')
    printf("f(x) = 3^x\n");
  /* Needs updating   $  */

  for (i=0;i<=n;i++) {		/* Enter data. */
    fprintf(stderr, "Enter x%d: ", i);
    scanf("%lf", &x[i]);
    if (ch != 'y') {		/* Use data points. */
      fprintf(stderr, "Enter f(%lg): ", x[i]);
      scanf("%lf", &Q[i][0]);
      fprintf(stderr, "\n");
    } else
      Q[i][0] = f(x[i]);	/* Use function. */
  }

  /*-------------------------------------------------------------------------*/
  printf("\n\n i\t  x(i) ");    	/* print out table header */
  for (i=0;i<=n;i++)
    printf("\t  Q(i,%d)", i);
  printf("\n-------------------");
  for (i=0;i<=n;i++)
    printf("---------------");
  printf("\n");
  /*-------------------------------------------------------------------------*/

  /* STEP #1 */
  for (i=1;i<=n;i++)
    for (j=1;j<=i;j++)
      Q[i][j] = ((X-x[i-j])*Q[i][j-1] - (X-x[i])*Q[i-1][j-1])
		/ (x[i] - x[i-j]);

  /* STEP #2*/
  for (i=0;i<=n;i++) {
    printf("% d\t % .2lf\t", i, x[i]);
    for (j=0;j<=i;j++)
      printf("% .9lf\t", Q[i][j]);
    printf("\n");
  }
  printf("\nP(%lg) = %.11lg\n", X, Q[n][n]);

}  /* STOP */

/*****************************************************************************/
double f(x)		/* Function to evaluate, f(x).  May be unnecessary. */
double x;
{
  return (pow(3.0, x));
}
/*****************************************************************************/
