/******************************************************************************
	1st Derivative Approximator (for functions) - Algorithm 4.0B1
*******************************************************************************

To approximate the derivative of f(x) at the point x0 for a given h.  Uses
Equation 4.2, 4.12, 4.13, 4.14, or 4.15.

INPUT the point x0, interval h, function f() and f_prime() (if known):

OUTPUT approximation f'(x0).

*******************************************************************************
*	Written by:  Harold A. Toomey, BYU, Summer 1989, v4.11		      *
******************************************************************************/

#include <stdio.h>		/* for scanf() and printf() */
#include <math.h>		/* for math functions */
#include "naa_util.h"		/* Numerical Analysis Algorithms Utilities. */
#define	PI	3.14159265358979323846264338327950288419716939937510582097

double f(), Eq4_2(), Eq4_12(), Eq4_13(), Eq4_14(), Eq4_15();


main()
{
  double x0, h, f_prime(), Dxf;
  int ch, Eqnum;

  get_title();

  printf("1st Derivative Approximator (for functions)");
  printf(" - Algorithm 4.0B1\n\n");
  fprintf(stderr, "1st Derivative Approximator (for functions)");
  fprintf(stderr, " - Algorithm 4.0B1\n\n");

  printf("f(x) = x*exp(x)\n\n");
  /* Needs updating  $  */

  do {

    fprintf(stderr, "Select desired formula:              ");
    fprintf(stderr, "     x-2, x-1, x0, x1, x2, x3, x4\n");
    fprintf(stderr, "  (1)   Equation 4.2   (2-Point Formula) ");
    fprintf(stderr, "            X   X\n");
    fprintf(stderr, "  (2)   Equation 4.12  (3-Point Formula) ");
    fprintf(stderr, "            X   X   X\n");
    fprintf(stderr, "  (3)   Equation 4.13  (3-Point Formula) ");
    fprintf(stderr, "       X        X\n");
    fprintf(stderr, "  (4)   Equation 4.14  (5-Point Formula) ");
    fprintf(stderr, "  X    X        X   X\n");
    fprintf(stderr, "  (5)   Equation 4.15  (5-Point Formula) ");
    fprintf(stderr, "            X   X   X   X   X\n");
    fprintf(stderr, "\n");
    scanf("%d", &Eqnum);

    switch (Eqnum) {
      case 1: printf("Using Equation 4.2  (Two-Point Formula) "); break;
      case 2: printf("Using Equation 4.12 (Three-Point Formula) "); break;
      case 3: printf("Using Equation 4.13 (Three-Point Formula) "); break;
      case 4: printf("Using Equation 4.14 (Five-Point Formula) "); break;
      case 5: printf("Using Equation 4.15 (Five-Point Formula) "); break;
      default: printf("Using Equation 4.13 (Three-Point Formula) "); break;
    }

    fprintf(stderr, "Enter point to evaluate, x0: ");
    scanf("%lf", &x0);

    fprintf(stderr, "Enter interval value, h (may be negative): ");
    scanf("%lf", &h);
    printf("with h = %lg:\n", h);

    switch (Eqnum) {
      case 1:  Dxf = Eq4_2(x0, h);  break;
      case 2:  Dxf = Eq4_12(x0, h);  break;
      case 3:  Dxf = Eq4_13(x0, h);  break;
      case 4:  Dxf = Eq4_14(x0, h);  break;
      case 5:  Dxf = Eq4_15(x0, h);  break;
      default: Dxf = Eq4_13(x0, h); break;
    }

    fprintf(stderr, "f'(%lg) = %lf\n", x0, Dxf);
    printf("f'(%lg) = %lf\n", x0, Dxf);
    printf("True value = %lf\n", f_prime(x0));
    printf("Error = %.2le\n\n", fabs(f_prime(x0) - Dxf));

    fprintf(stderr, "\nWould you like to try another method, h,");
    fprintf(stderr, " or x0? (y/n): ");
    ch = getchar();
    ch = tolower(getchar());

  } while (ch == 'y');

}

/*****************************************************************************/
double Eq4_2(x0, h)		/* Equation 4.2, p.146 */
double x0, h;
{
  return ((f(x0 + h) - f(x0)) / h);
}

/*****************************************************************************/
double Eq4_12(x0, h)		/* Equation 4.12, p.149 */
double x0, h;
{
  return ((-3.0*f(x0) + 4.0*f(x0 + h) - f(x0 + 2.0*h)) / (2.0*h));
}

/*****************************************************************************/
double Eq4_13(x0, h)		/* Equation 4.13, p.149 */
double x0, h;
{
  return ((f(x0 + h) - f(x0 - h)) / (2.0*h));
}

/*****************************************************************************/
double Eq4_14(x0, h)		/* Equation 4.14, p.150 */
double x0, h;
{
  return ((f(x0 - 2*h) - 8.0*f(x0 - h) + 8.0*f(x0 + h) - f(x0 + 2*h))
	 / (12.0*h));
}

/*****************************************************************************/
double Eq4_15(x0, h)		/* Equation 4.15, p.150 */
double x0, h;
{
  return ((-25.0*f(x0) + 48.0*f(x0 + h) - 36.0*f(x0 + 2*h) + 16.0*f(x0 + 3*h)
	- 3.0*f(x0 + 4*h)) / (12.0*h));
}

/*****************************************************************************/
/*****************************************************************************/
double f(x)			/* Function to evaluate. */
double x;
{
  return (x*exp(x));
}

/*****************************************************************************/
double f_prime(x)		/* Actual derivative, if known. */
double x;
{
  return ((x+1.0)*exp(x));
}
/*****************************************************************************/
