/******************************************************************************
		Newton-Cotes Formulas for Integrals - Algorithm 4.1D
*******************************************************************************
                                [b
To approximate the integral I = |  f(x) dx:
                                ]a
INPUT endpoints a, b; formula selection, type; the function f():

OUTPUT approximation I to I.

*******************************************************************************
*	Written by:  Harold A. Toomey, BYU, Summer 1989, v4.11		      *
******************************************************************************/

#include <stdio.h>		/* for scanf() and printf() */
#include <math.h>		/* for math functions */
#include "naa_util.h"		/* Numerical Analysis Algorithms Utilities. */
#define	PI	3.14159265358979323846264338327950288419716939937510582097

double a, b, f();


main()
{
  int ch, type;

  get_title();

  printf("Newton-Cotes Formulas for Integrals - Algorithm 4.1D\n\n");
  fprintf(stderr, "Newton-Cotes Formulas for Integrals - Algorithm 4.1D\n\n");

  printf("f(x) = sin(x)");
  /* Needs updating  $  */

  fprintf(stderr, "Enter endpoint a: ");
  scanf("%lf", &a);

  fprintf(stderr, "Enter endpoint b: ");
  scanf("%lf", &b);
  printf("  from %lg to %lg.\n\n", a, b);

  do {

  fprintf(stderr,"\nSelect desired formula:\n");
  fprintf(stderr,"  Closed Newton-Cotes Formulas:\n");
  fprintf(stderr,"    (1)   (n=1)  Trapezoidal Rule             Eq. (4.40)\n");
  fprintf(stderr,"    (2)   (n=2)  Simpson's Rule               Eq. (4.41)\n");
  fprintf(stderr,"    (3)   (n=3)  Simpson's Three-eighth Rule  Eq. (4.42)\n");
  fprintf(stderr,"    (4)   (n=4)                               Eq. (4.43)\n");
  fprintf(stderr,"  Open Newton-Cotes Formulas:\n");
  fprintf(stderr,"    (5)   (n=0)  Midpoint Rule                Eq. (4.46)\n");
  fprintf(stderr,"    (6)   (n=1)                               Eq. (4.47)\n");
  fprintf(stderr,"    (7)   (n=2)                               Eq. (4.48)\n");
  fprintf(stderr,"    (8)   (n=3)                               Eq. (4.49)\n");
  fprintf(stderr,"    (9)   All of the above (1-8).\n");
  fprintf(stderr, "\n");
  scanf("%d", &type);

    if ((type < 1) || (type > 9))
      naaerror("Invalid selection.  Use (1-9 only).");

    if (type == 9) {		/* Try ALL methods */
      for (type=1;type<=8;type++)
        Evaluate(type);
      exit(1);			/* ... then exit program. */
    } else			/* Try just one method at a time. */
      Evaluate(type);

    fprintf(stderr, "\n\nI'm all done!\n");
    fprintf(stderr, "Would you like to try another method? (y/n): ");
    ch = getchar();
    ch = tolower(getchar());

  } while (ch == 'y');

}

/*****************************************************************************/
Evaluate(type)		/* Evaluates a specific integral formula. */
int type;
{
  double Eq4_40(), Eq4_41(), Eq4_42(), Eq4_43();
  double Eq4_46(), Eq4_47(), Eq4_48(), Eq4_49();
  double I;

  switch (type) {
    case 1: printf("Using Eq. (4.40)  Trapezoidal Rule\n\n");
            I = Eq4_40();
            break;
    case 2: printf("Using Eq. (4.41)  Simpson's Rule\n\n"); 
            I = Eq4_41();
            break;
    case 3: printf("Using Eq. (4.42)  Simpson's Three-eights Rule\n\n"); 
            I = Eq4_42();
            break;
    case 4: printf("Using Eq. (4.43)  \n\n"); 
            I = Eq4_43();
            break;
    case 5: printf("Using Eq. (4.46)  Midpoint Rule\n\n"); 
            I = Eq4_46();
            break;
    case 6: printf("Using Eq. (4.47)  \n\n"); 
            I = Eq4_47();
            break;
    case 7: printf("Using Eq. (4.48)  \n\n"); 
            I = Eq4_48();
            break;
    case 8: printf("Using Eq. (4.49)  \n\n"); 
            I = Eq4_49();
            break;
  }

  printf("    [%lg\n", b);
  printf("I = |  f(x) dx = %.11lg\n", I);
  printf("    ]%lg\n\n", a);

}

/*****************************************************************************/
double Eq4_40()		/* Equation 4.40, p.169 (Trapezoidal Rule) */
{
  return (((b-a)/2.0) * (f(a) + f(b)));
}

/*****************************************************************************/
double Eq4_41()		/* Equation 4.41, p.169 (Simpson's Rule) */
{
  double h;

  h = (b-a) / 2.0;
  return ((h/3.0) * (f(a) + 4.0*f(a+h) + f(b)));
}

/*****************************************************************************/
double Eq4_42()		/* Equation 4.42, p.169 (Simpson's Three-eight Rule) */
{
  double h;

  h = (b-a) / 3.0;
  return (((3.0*h)/8.0) * (f(a) + 3.0*f(a+h) + 3.0*f(a+2*h) + f(b)));
}

/*****************************************************************************/
double Eq4_43()		/* Equation 4.43, p.169 */
{
  double h;

  h = (b-a) / 4.0;
  return (((2.0*h)/45.0) * (7.0*f(a) + 32.0*f(a+h) + 12.0*f(a+2*h)
	+ 32.0*f(a+3*h) + 7.0*f(b)));
}

/*****************************************************************************/
double Eq4_46()		/* Equation 4.46, p.170 (Midpoint Rule) */
{
  double h;

  h = (b-a) / 2.0;
  return (2.0*h * f(a+h));
}

/*****************************************************************************/
double Eq4_47()		/* Equation 4.47, p.171 */
{
  double h;

  h = (b-a) / 3.0;
  return (((3.0*h)/2.0) * (f(a+h) + f(a+2*h)));
}

/*****************************************************************************/
double Eq4_48()		/* Equation 4.48, p.171 */
{
  double h;

  h = (b-a) / 4.0;
  return (((4.0*h)/3.0) * (2.0*f(a+h) - f(a+2*h) + 2.0*f(a+3*h)));
}

/*****************************************************************************/
double Eq4_49()		/* Equation 4.49, p.171 */
{
  double h;

  h = (b-a) / 5.0;
  return (((5.0*h)/24.0) * (11.0*f(a+h) + f(a+2*h) + f(a+3*h) +11.0*f(a+4*h)));
}

/*****************************************************************************/
/*****************************************************************************/
double f(x)		/* Function to evaluate. */
double x;
{
  return (sin(x));
}
/*****************************************************************************/
