/******************************************************************************
	Adams-Bashforth (all four) and Milne's Methods - Algorithm 5.4B
*******************************************************************************

To approximate the solution of the initial-value problem
	y' = f(t,y),  a <= t <= b,  y(a) = alpha,
at (N+1) equally spaced numbers in the interval [a,b]:

INPUT endpoints a, b; integer N or interval h; initial conditions alpha,
alpha1, alpha2, alpha3, and alpha4 where applicable; the function f(); which
method(s) to try:

OUTPUT approximation w to y at the (N+1) values of t.

*******************************************************************************
*	Written by:  Harold A. Toomey, BYU, Summer 1989, v4.11		      *
******************************************************************************/

#include <stdio.h>		/* for scanf() and printf() */
#include <math.h>		/* for math functions */
#include "naa_util.h"		/* Numerical Analysis Algorithms Utilities. */
#define	PI	3.14159265358979323846264338327950288419716939937510582097

double a, b, h=0.0, alpha[5], f();
int N;


main()
{
  int i, ch, OK=0, type, HAVEID, HAVEALPHA;
  double t, Runge_Kutta_4();

  get_title();

  printf("Adams-Bashforth (all four) and Milne's Methods");
  printf(" - Algorithm 5.4B\n\n");
  fprintf(stderr, "Adams-Bashforth (all four) and Milne's Methods");
  fprintf(stderr, " - Algorithm 5.4B\n\n");

  printf("y' = f(t,y) = -y + t + 1\n\n");
  /* Needs updating         $          */

  HAVEID = 0;		/* Flag to indicate initial data status. */
  HAVEALPHA = 0;	/* Flag to indicate alpha status. */

  do {

    fprintf(stderr,"\nSelect desired method:\n\n");
    fprintf(stderr,"    (1)   Runge-Kutta (Order Four) Method   (5.39)\n\n");

    fprintf(stderr,"    (2)   Adams-Bashforth Two-Step Method   (5.63)\n");
    fprintf(stderr,"    (3)   Adams-Bashforth Three-Step Method (5.64)\n");
    fprintf(stderr,"    (4)   Adams-Bashforth Four-Step Method  (5.65)\n");
    fprintf(stderr,"    (5)   Adams-Bashforth Five-Step Method  (5.66)\n\n");

    fprintf(stderr,"    (6)   Milne's Method                    (5.70)\n\n");

    fprintf(stderr,"    (7)   All Adams-Bashforth Methods above (2-5) with\n");
    fprintf(stderr,"          starting values from the Runge-Kutta (Order\n");
    fprintf(stderr,"          Four) Method.\n\n");

    fprintf(stderr,"    (8)   ALL of the above (1-7).\n");
    fprintf(stderr, "\n");
    scanf("%d", &type);

      if ((type < 1) || (type > 8))
        naaerror("Invalid selection.  Use (1-8 only).");

      if (HAVEID == 0) {	/* Get initial data ONCE. */
        HAVEID = 1;

      fprintf(stderr, "Enter endpoint a: ");
      scanf("%lf", &a);
      fprintf(stderr, "Enter endpoint b: ");
      scanf("%lf", &b);
      printf("Solving for points on interval [%lg,%lg].\n", a, b);

      do {
        fprintf(stderr, "Do you want to enter N or h? ");
        ch = getchar();		/* Filters out Carriage Return/Linefeed. */
        ch = tolower(getchar());
        if (ch == 'n') {
          fprintf(stderr, "Enter number of subintervals N: ");
          scanf("%d", &N);
          h = (b-a)/N;
          OK = 1;
        }
        if (ch == 'h') {
          fprintf(stderr, "Enter interval number h: ");
          scanf("%lf", &h);
          N = (int) ceil((b-a)/h);
          OK = 1;
        }
        if (OK==0)
          fprintf(stderr, "\tTRY AGAIN.\n");
      } while (OK==0);

      printf("\nNumber of subintervals N = %d\n", N);
      printf("Interval number h = %lg\n", h);
    }

    if ((HAVEALPHA == 0) && (((type != 1) && (type != 7)) || (type == 8))) {
      fprintf(stderr, "\nEnter initial conditions ");
      fprintf(stderr, "(enter 0 for each alpha NOT needed): \n");
      printf("\n");
      for (i=0;i<=4;i++) {
        fprintf(stderr, "\ty(%lg) = alpha%d: ", a+i*h, i);
        scanf("%lf", &alpha[i]);
        printf("y(%lg) = alpha%d = % .11lf\n", a+i*h, i, alpha[i]);
      }
      printf("\n");
      HAVEALPHA = 1;
    }
				/* For RK4's alpha. */
    if ((HAVEALPHA == 0) && ((type == 1) || (type >= 7))) {
      fprintf(stderr, "\ty(%lg) = alpha: ", a);
      scanf("%lf", &alpha[0]);
      printf("y(%lg) = alpha = % .11lf\n", a, alpha[0]);
    }

    if (type == 8) {		/* Try ALL methods */
      for (type=1;type<=6;type++)
        Evaluate(type);
      type = 7;
      printf("\nUsing all 4 Adams-Bashforth Methods with starting\n");
      printf("values from the Runge-Kutta (Order Four) Method:\n");
    }

    if (type == 7) {	/* Try all Adams-Bashforth methods */
      for (i=0;i<=3;i++) {	/* Determine alphas from RK4 Method. */
        t = a + i*h;
        alpha[i+1] = Runge_Kutta_4(t,alpha[i]);
      }
      for (type=2;type<=5;type++)
        Evaluate(type);
      HAVEID = 0;		/* Reset flags. */
      HAVEALPHA = 0;

    } else			/* Try just one method at a time. */
      Evaluate(type);

    fprintf(stderr, "\n\nI'm all done!\n");
    fprintf(stderr, "Would you like to try another method? (y/n): ");
    ch = getchar();
    ch = tolower(getchar());

  } while (ch == 'y');

}

/*****************************************************************************/
Evaluate(type)		/* Evaluates a specific derivative formula. */
int type;
{
  double Runge_Kutta_4(), Adams_Bashforth_2(), Adams_Bashforth_3();
  double Adams_Bashforth_4(), Adams_Bashforth_5(), Milne();
  double t, w[5], W;
  int i, j, start;

  switch (type) {
    case 1: printf("\nUsing Runge-Kutta (Order Four) Method:\n");
            start = 1;
            break;
    case 2: printf("\nUsing Adams-Bashforth Two-Step Method:\n");
            start = 2;
            break;
    case 3: printf("\nUsing Adams-Bashforth Three-Step Method:\n");
            start = 3;
            break;
    case 4: printf("\nUsing Adams-Bashforth Four-Step Method:\n");
            start = 4;
            break;
    case 5: printf("\nUsing Adams-Bashforth Five-Step Method:\n");
            start = 5;
            break;
    case 6: printf("\nUsing Milne's Method:\n");
            start = 4;
            break;
  }

  printf("\n t\t  w = y\n-----------------------\n");
  t = a;

  for (i=0;i<start;i++) {	/* Print given initial values. */
    t = a + i*h;
    w[i] = alpha[i];
    printf("% .2lf\t% .11lf\n", t, w[i]);
  }

  for (i=start;i<=N;i++) {
    switch (type) {		/* compute wi */
      case 1:  W = Runge_Kutta_4(t,w[0]);  break;
      case 2:  W = Adams_Bashforth_2(t,w,start-1);  break;
      case 3:  W = Adams_Bashforth_3(t,w,start-1);  break;
      case 4:  W = Adams_Bashforth_4(t,w,start-1);  break;
      case 5:  W = Adams_Bashforth_5(t,w,start-1);  break;
      case 6:  W = Milne(t,w,start-1);  break;
    }

    t = a + i*h;		/* compute ti */

    if (type != 1)	/* Prepare w[] for next run. */
      for (j=0;j<start-1;j++)
        w[j] = w[j+1];
    w[start-1] = W;

    printf("% .2lf\t% .11lf\n", t, W);
  }
  printf("\n");
}

/*****************************************************************************/
double Runge_Kutta_4(t,w)	/* Runge-Kutta (Order Four) Method */
double t, w;
{
  double k1, k2, k3, k4;

  k1 = h*f(t, w);
  k2 = h*f(t + h/2, w + k1/2);
  k3 = h*f(t + h/2, w + k2/2);
  k4 = h*f(t + h, w + k3);

  return (w + (k1 + 2*k2 + 2*k3 + k4)/6);
}

/*****************************************************************************/
double Adams_Bashforth_2(t,w,i)	/* Adams-Bashforth Two-Step Method */
double t, w[5];
int i;
{
  return (w[i] + (h/2)*(3*f(t,w[i]) - f(t-h,w[i-1])));
}

/*****************************************************************************/
double Adams_Bashforth_3(t,w,i)	/* Adams-Bashforth Three-Step Method */
double t, w[5];
int i;
{
  return (w[i] + (h/12)*(23*f(t,w[i]) - 16*f(t-h,w[i-1]) + 5*f(t-2*h,w[i-2])));
}

/*****************************************************************************/
double Adams_Bashforth_4(t,w,i)	/* Adams-Bashforth Four-Step Method */
double t, w[5];
int i;
{
  return (w[i] + (h/24)*(55*f(t,w[i]) - 59*f(t-h,w[i-1])
	 + 37*f(t-2*h,w[i-2]) - 9*f(t-3*h,w[i-3])));
}

/*****************************************************************************/
double Adams_Bashforth_5(t,w,i)	/* Adams-Bashforth Five-Step Method */
double t, w[5];
int i;
{
  return (w[i] + (h/720)*(1901*f(t,w[i]) - 2774*f(t-h,w[i-1])
	+ 2616*f(t-2*h,w[i-2]) - 1274*f(t-3*h,w[i-3])
	+ 251*f(t-4*h,w[i-4])));
}

/*****************************************************************************/
double Milne(t,w,i)	/* Milne's Method */
double t, w[5];
int i;
{
  return (w[i-3] + (4*h/3)*(2*f(t,w[i]) - f(t-h,w[i-1])
	+ 2*f(t-2*h,w[i-2])));
}

/*****************************************************************************/
/*****************************************************************************/
double f(t,y)		/* Function f(). */
double t, y;
{
  return (-y + t + 1);
}
/*****************************************************************************/
