/******************************************************************************
			Jacobi Iterative - Algorithm 7.1
*******************************************************************************

To solve AX=B given an initial approximation X0:

INPUT the number of equations and unknowns n; the entries aij, 1 <= i,j <= n of
the matrix A; the entire bi, 1 <= i <= n of the inhomogeneous term B; the entries
X0i, 1 <= i <= n of X0 = x(0); tolerance TOL; maximum number of iterations N.

OUTPUT the approximate solution x1,...,xn or a message that the number of
iterations was exceeded.

*******************************************************************************
*	Written by:  Harold A. Toomey, BYU, Summer 1989, v4.11		      *
******************************************************************************/

#include <stdio.h>		/* for scanf() and printf() */
#include <math.h>		/* for fabs() */
#include "naa_util.h"		/* Numerical Analysis Algorithms Utilities. */

int n;				/* Number of equations and unknowns. */


main()
{
  double **A, *X, *B, *X0, *TEMP, TOL, infinite_norm();
  int i, j, k, N;

  get_title();

  printf("Jacobi Iterative - Algorithm 7.1\n\n");
  fprintf(stderr, "Jacobi Iterative - Algorithm 7.1\n\n");

  fprintf(stderr, "Enter maximum number of iterations, N: ");
  scanf("%d", &N);
  printf("Maximum number of iterations: %d\n", N);

  fprintf(stderr, "Enter the tolerance, TOL (1.0e-4): ");
  scanf("%lf", &TOL);
  printf("Tolerance = %lg\n", TOL);

  fprintf(stderr, "Enter the dimension, n, of the matrix A: ");
  scanf("%d", &n);
  if (n <= 0)
    naaerror("n must be > 0.");	/* ERROR. */
  printf("System has %d equations and %d unknowns.\n\n", n, n);

  A = dmatrix(1,n,1,n);		/* Matrix A */
  X = dvector(1,n);		/* solution vector */
  B = dvector(1,n);		/* Matrix B */
  X0 = dvector(1,n);		/* initial solution vector */
  TEMP = dvector(1,n);		/* Temporary matrix for error */

  fprintf(stderr, "Enter the coefficients for matrix A:\n");
  printf("A = ");
  for (i=1;i<=n;i++) {
    printf("[ ");
    for (j=1;j<=n;j++) {
      fprintf(stderr, "\tA[%d][%d] = ", i, j);
      scanf("%lf", &A[i][j]);
      printf("% 9.9lg ", A[i][j]);
    }
    printf("]\n    ");
  }
  printf("\n");

  fprintf(stderr, "Enter the coefficients for matrix B:\n");
  printf("B = [ ");
  for (i=1;i<=n;i++) {
    fprintf(stderr, "\tB[%d] = ", i);
    scanf("%lf", &B[i]);
    printf("% 9.9lg ", B[i]);
  }
  printf("]t\n\n");

  fprintf(stderr, "Enter initial approximation vector X0:\n");
  printf("X0 = [ ");
  for (i=1;i<=n;i++) {
    fprintf(stderr, "\tX0[%d] = ", i);
    scanf("%lf", &X0[i]);
    printf("% 9.9lg ", X0[i]);
  }
  printf("]t\n");

  /*-------------------------------------------------------------------------*/
  printf("\n k\t");		/* print out table header */
  for (i=1;i<=n;i++)
    printf(" X[%d]\t\t", i);
  printf("\n----------");
  for (i=1;i<=n;i++)
    printf("---------------");
  printf("\n");
  /*-------------------------------------------------------------------------*/

  printf("% d\t", 0);
  for (i=1;i<=n;i++)
    printf("% .9lf\t", X0[i]);
  printf("\n");

  /* STEP #1 */
  k = 1;

  /* STEP #2 */
  while (k <= N) {		/* Do Steps 3-6. */

    /* STEP #3 */
    for (i=1;i<=n;i++) {
      X[i] = 0.0;
      for (j=1;j<=n;j++)
        if (j != i)
          X[i] -= A[i][j] * X0[j];
      X[i] = (X[i] + B[i]) / A[i][i];
    }

    /* STEP #4 */
    printf("% d\t", k);
    for (i=1;i<=n;i++)
      printf("% .9lf\t", X[i]);
    printf("\n");

    for (i=1;i<=n;i++)
      TEMP[i] = X[i] - X0[i];
    if (fabs(infinite_norm(TEMP)) < TOL)
      exit (1);			/* STOP - Procedure completed successfully. */

    /* STEP #5 */
    k++;

    /* STEP #6 */
    for (i=1;i<=n;i++)
      X0[i] = X[i];
  }

  /* STEP #7 */			/* Procedure completed unsuccessfully. */
  printf("\nMaximum number of iterations (%d) exceeded.\n", N);

}  /* STOP */

/****************************************************************************/
double infinite_norm(X)
double *X;
{
  double max = 0.0;
  int i;

  for (i=1;i<=n;i++)
    if (fabs(X[i]) > fabs(max))
      max = X[i];
  return (max);
}
/****************************************************************************/
