/******************************************************************************
	"Numerical Analysis Algorithms in C" Utilities II v4.11
******************************************************************************/

/*  For separate compilation, remove the comment symbols below. */
/* #include "complex.h"*/	/* Needed for struct complex. */

/* #include <stdlib.h>		/* For calloc(). */


/* NEW STANDARD PROTOTYPING */
/*
int ***icube(int, int, int, int, int, int);

double ***dcube(int, int, int, int, int, int);

long double ***ldcube(int, int, int, int, int, int);
long double **ldmatrix(int, int, int, int);
long double *ldvector(int, int);

fcomplex ***ccube(int, int, int, int, int, int);
fcomplex **cmatrix(int, int, int, int);
fcomplex *cvector(int, int);

void free_icube(int ***, int, int, int, int, int, int);
void free_dcube(double ***, int, int, int, int, int, int);
void free_ldcube(long double ***, int, int, int, int, int, int);
void free_ldmatrix(long double **, int, int, int, int);
void free_ldvector(long double *, int, int);
void free_ccube(fcomplex ***, int, int, int, int, int, int);
void free_cmatrix(fcomplex **, int, int, int, int);
void free_cvector(fcomplex *, int, int);
*/


/* OLD STANDARD PROTOTYPING */
int ***icube();

double ***dcube();

long double ***ldcube();
long double **ldmatrix();
long double *ldvector();

fcomplex ***ccube();
fcomplex **cmatrix();
fcomplex *cvector();

void free_icube();
void free_dcube();
void free_ldcube();
void free_ldmatrix();
void free_ldvector();
void free_ccube();
void free_cmatrix();
void free_cvector();


/*****************************************************************************/
int ***icube(a,b,c,d,e,f)
int a,b,c,d,e,f;
/* Allocates an integer cube with range [a..b][c..d][e..f]. */
/* Like dmatrix() but for three dimensional arrays. */
{
  int i,j;
  int ***m;

  m=(int ***) calloc((unsigned) (b-a+1), sizeof(int **));
  if (!m) exit(-1);
  m -= a;

  for (i=a;i<=b;i++) {
    m[i]=(int **) calloc((unsigned) (d-c+1), sizeof(int *));
    if (!m[i]) exit(-1);
    m[i] -= c;
  }

  for (i=a;i<=b;i++)
    for (j=c;j<=d;j++) {
      m[i][j]=(int *) calloc((unsigned) (f-e+1), sizeof(int));
      if (!m[i][j]) exit(-1);
      m[i][j] -= e;
    }

  return (m);		/* return pointer to array of pointers to rows */
}

/*****************************************************************************/
double ***dcube(a,b,c,d,e,f)
int a,b,c,d,e,f;
/* Allocates a double cube with range [a..b][c..d][e..f]. */
/* Like dmatrix() but for three dimensional arrays. */
{
  int i,j;
  double ***m;

  m=(double ***) calloc((unsigned) (b-a+1), sizeof(double **));
  if (!m) exit(-1);
  m -= a;

  for (i=a;i<=b;i++) {
    m[i]=(double **) calloc((unsigned) (d-c+1), sizeof(double *));
    if (!m[i]) exit(-1);
    m[i] -= c;
  }

  for (i=a;i<=b;i++)
    for (j=c;j<=d;j++) {
      m[i][j]=(double *) calloc((unsigned) (f-e+1), sizeof(double));
      if (!m[i][j]) exit(-1);
      m[i][j] -= e;
    }

  return (m);		/* return pointer to array of pointers to rows */
}

/*****************************************************************************/
long double ***ldcube(a,b,c,d,e,f)
int a,b,c,d,e,f;
/* Allocates a long double cube with range [a..b][c..d][e..f]. */
/* Like ldmatrix() but for three dimensional arrays. */
{
  int i,j;
  long double ***m;

  m=(long double ***) calloc((unsigned) (b-a+1), sizeof(long double **));
  if (!m) exit(-1);
  m -= a;

  for (i=a;i<=b;i++) {
    m[i]=(long double **) calloc((unsigned) (d-c+1), sizeof(long double *));
    if (!m[i]) exit(-1);
    m[i] -= c;
  }

  for (i=a;i<=b;i++)
    for (j=c;j<=d;j++) {
      m[i][j]=(long double *) calloc((unsigned) (f-e+1), sizeof(long double));
      if (!m[i][j]) exit(-1);
      m[i][j] -= e;
    }

  return (m);		/* return pointer to array of pointers to rows */
}

/*****************************************************************************/
long double **ldmatrix(a,b,c,d)
int a,b,c,d;
/* allocates a long double matrix with range [a..b][c..d]. */
{
  int i;
  long double **m;

  /* allocate pointers to rows */
  m=(long double **) calloc((unsigned) (b-a+1), sizeof(long double*));
  if (!m) nrerror("allocation failure 1 in dmatrix()");
  m -= a;

  for (i=a;i<=b;i++) {	/* allocate rows and set pointers to them */
    m[i]=(long double *) calloc((unsigned) (d-c+1), sizeof(long double));
    if (!m[i]) nrerror("allocation failure 2 in dmatrix()");
    m[i] -= c;
  }

  return (m);		/* return pointer to array of pointers to rows */
}

/*****************************************************************************/
long double *ldvector(a,b)
int a,b;
/* allocates a long double vector with range [a..b]. */
{
  long double *v;

  v=(long double *)calloc((unsigned) (b-a+1), sizeof(long double));
  if (!v) nrerror("allocation failure in dvector()");
  return (v-a);
}

/*****************************************************************************/
fcomplex ***ccube(a,b,c,d,e,f)
int a,b,c,d,e,f;
/* Allocates a complex cube with range [a..b][c..d][e..f]. */
/* Like cmatrix() but for three dimensional arrays. */
{
  int i,j;
  fcomplex ***m;

  m=(fcomplex ***) calloc((unsigned) (b-a+1), sizeof(fcomplex **));
  if (!m) exit(-1);
  m -= a;

  for (i=a;i<=b;i++) {
    m[i]=(fcomplex **) calloc((unsigned) (d-c+1), sizeof(fcomplex *));
    if (!m[i]) exit(-1);
    m[i] -= c;
  }

  for (i=a;i<=b;i++)
    for (j=c;j<=d;j++) {
      m[i][j]=(fcomplex *) calloc((unsigned) (f-e+1), sizeof(fcomplex));
      if (!m[i][j]) exit(-1);
      m[i][j] -= e;
    }

  return (m);		/* return pointer to array of pointers to rows */
}

/*****************************************************************************/
fcomplex **cmatrix(a,b,c,d)
int a,b,c,d;
/* allocates a complex matrix with range [a..b][c..d]. */
{
  int i;
  fcomplex **m;

  /* allocate pointers to rows */
  m=(fcomplex **) calloc((unsigned) (b-a+1), sizeof(fcomplex*));
  if (!m) nrerror("allocation failure 1 in dmatrix()");
  m -= a;

  for (i=a;i<=b;i++) {	/* allocate rows and set pointers to them */
    m[i]=(fcomplex *) calloc((unsigned) (d-c+1), sizeof(fcomplex));
    if (!m[i]) nrerror("allocation failure 2 in dmatrix()");
    m[i] -= c;
  }

  return (m);		/* return pointer to array of pointers to rows */
}

/*****************************************************************************/
fcomplex *cvector(a,b)
int a,b;
/* allocates a complex vector with range [a..b]. */
{
  fcomplex *v;

  v = (fcomplex *) malloc((unsigned) (b-a+1) * sizeof(fcomplex));
  if (!v) nrerror("allocation failure in cvector()");
  return (v-a);
}

/*****************************************************************************/
void free_icube(m,a,b,c,d,e,f)
int ***m;
int a,b,c,d,e,f;	/* (variable f is never used.) */
/* frees a integer cube allocated with icube(). */
{
  int i, j;

  for(i=b;i>=a;i--)
    for(j=d;j>=c;j--)
      free((char*) (m[i][j]+e));
  for(i=b;i>=a;i--)
    free((char*) (m[i]+c));
  free((char*) (m+a));
}

/*****************************************************************************/
void free_dcube(m,a,b,c,d,e,f)
double ***m;
int a,b,c,d,e,f;	/* (variable f is never used.) */
/* frees a double cube allocated with dcube(). */
{
  int i, j;

  for(i=b;i>=a;i--)
    for(j=d;j>=c;j--)
      free((char*) (m[i][j]+e));
  for(i=b;i>=a;i--)
    free((char*) (m[i]+c));
  free((char*) (m+a));
}

/*****************************************************************************/
void free_ldcube(m,a,b,c,d,e,f)
long double ***m;
int a,b,c,d,e,f;	/* (variable f is never used.) */
/* frees a long double cube allocated with ldcube(). */
{
  int i, j;

  for(i=b;i>=a;i--)
    for(j=d;j>=c;j--)
      free((char*) (m[i][j]+e));
  for(i=b;i>=a;i--)
    free((char*) (m[i]+c));
  free((char*) (m+a));
}

/*****************************************************************************/
void free_ldmatrix(m,a,b,c,d)
long double **m;
int a,b,c,d;		/* (variable d is never used.) */
/* frees a long double matrix allocated with ldmatrix(). */
{
  int i;

  for(i=b;i>=a;i--)
    free((char*) (m[i]+c));
  free((char*) (m+a));
}

/*****************************************************************************/
void free_ldvector(v,a,b)
long double *v;
int a,b;		/* (variable b is never used.) */
/* frees a long double vector allocated by ldvector(). */
{
  free((char*) (v+a));
}

/*****************************************************************************/
void free_ccube(m,a,b,c,d,e,f)
fcomplex ***m;
int a,b,c,d,e,f;	/* (variable f is never used.) */
/* frees a complex cube allocated with ccube(). */
{
  int i, j;

  for(i=b;i>=a;i--)
    for(j=d;j>=c;j--)
      free((char*) (m[i][j]+e));
  for(i=b;i>=a;i--)
    free((char*) (m[i]+c));
  free((char*) (m+a));
}

/*****************************************************************************/
void free_cmatrix(m,a,b,c,d)
fcomplex **m;
int a,b,c,d;		/* (variable d is never used.) */
/* frees a complex matrix allocated with cmatrix(). */
{
  int i;

  for(i=b;i>=a;i--)
    free((char*) (m[i]+c));
  free((char*) (m+a));
}

/*****************************************************************************/
void free_cvector(v,a,b)
fcomplex *v;
int a,b;		/* (variable b is never used.) */
/* frees a complex vector allocated by cvector(). */
{
  free((char*) (v+a));
}

/******************************************************************************
*	Written by:  Harold A. Toomey, BYU, Summer 1989, v4.11		      *
******************************************************************************/
